#include "STM8S_SW_I2C.h"

unsigned int I2C_DelayValue = 1;
I2C_Result I2C__Start (void);
void I2C__ReStart (void);
void I2C__Stop (void);
unsigned char I2C__Clock(void);
unsigned char I2C__Write(unsigned char data);
unsigned char I2C__Read (unsigned char Acknowledgment);
void I2C_Delay (void);

void I2C__Init (unsigned long F_OSC_hz, unsigned long F_i2c_hz)
{
  GPIO_Init(I2C_SDA, GPIO_MODE_OUT_OD_HIZ_FAST);
  GPIO_Init(I2C_SCL, GPIO_MODE_OUT_OD_HIZ_FAST);
  //I2C_DelayValue = (u32)F_OSC_hz / (u32)F_i2c_hz;
}

I2C_Result I2C__Start (void)
{
  PIN_ON(I2C_SDA);
  PIN_ON(I2C_SCL);
  I2C_Delay();
  if (PIN_SYG(I2C_SDA) & PIN_SYG(I2C_SCL) == 0) return I2C_BUS_BUSY;
  PIN_OFF(I2C_SDA);
  I2C_Delay();
  PIN_OFF(I2C_SCL);
  I2C_Delay();
  return I2C_SUCCESS;
}

void I2C__ReStart (void)
{
  PIN_ON(I2C_SDA);
  PIN_ON(I2C_SCL);
  I2C_Delay();
  PIN_OFF(I2C_SDA);
  I2C_Delay();
  PIN_OFF(I2C_SCL);
}

void I2C__Stop (void)
{
  I2C_Delay();
  PIN_OFF(I2C_SDA);
  PIN_ON(I2C_SCL);
  I2C_Delay();
  PIN_ON(I2C_SDA);
  I2C_Delay();
}

unsigned char I2C__Clock(void)
{
  unsigned char Result; 
  PIN_ON(I2C_SCL);
  I2C_Delay();
  Result = PIN_SYG(I2C_SDA);
  PIN_OFF(I2C_SCL);
  I2C_Delay();
  return Result;  
}

unsigned char I2C__Write(unsigned char data)
{
  unsigned char mask = 0x80;
  while (mask)
  {
    if(data & mask)
     PIN_ON(I2C_SDA);
    else
     PIN_OFF(I2C_SDA);   
    I2C__Clock();  
    mask >>= 1;
  }
  PIN_ON(I2C_SDA);
  return I2C__Clock();
}

unsigned char I2C__Read (unsigned char Acknowledgment)
{
  unsigned char data = 0, mask = 0x80;        
  while(mask)
  {
    if (I2C__Clock())
      data |= mask;
    mask >>= 1;
  }
  if (Acknowledgment)
  {
    PIN_OFF(I2C_SDA);
    I2C__Clock();
    PIN_ON(I2C_SDA);
  }
  else
  {
    PIN_ON(I2C_SDA);
    I2C__Clock();
  }
  return data;
}


I2C_Result I2C__Read_Page      (unsigned char I2c_Address, 
                                unsigned char Mem_Address, 
                                unsigned char * pBuffer, 
                                unsigned char pBufferSize)
{
  I2C_Result Result = I2C__Start();
  if (Result) return Result;
  Result = I2C_ADD_NOT_EXIST;
  if (I2C__Write(I2c_Address) == ACK)
    if (I2C__Write(Mem_Address) == ACK)
    {
      I2C__ReStart();
      if (I2C__Write(I2c_Address | 1) == ACK)
      {
        while (pBufferSize--)
          *pBuffer++ = I2C__Read(pBufferSize ? 1 : 0);
        Result = I2C_SUCCESS;
      }
    }
  I2C__Stop();
  return Result;
}

I2C_Result I2C__Read_FullPage (unsigned char I2c_Address, 
                               unsigned char * Reg_Address,
                               unsigned char Reg_Address_Len,
                               unsigned char * pBuffer, 
                               unsigned char pBufferSize)
{
  I2C_Result Result = I2C__Start();
  if (Result) return Result;
  Result = I2C_ADD_NOT_EXIST;
  if (I2C__Write(I2c_Address & 0xFE) == ACK)
  {
    Result = (I2C_Result)ACK;
    while(Reg_Address_Len-- && Result == ACK)
      Result = (I2C_Result)I2C__Write(* Reg_Address++);
      I2C__ReStart();
      if (I2C__Write(I2c_Address | 0x01) == ACK)
      {
        while (pBufferSize--)
          *pBuffer++ = I2C__Read(pBufferSize ? 1 : 0);
        Result = I2C_SUCCESS;
      }   
  }
  I2C__Stop();
  return Result;
}

I2C_Result I2C__Write_Page      (unsigned char I2c_Address, 
                                 unsigned char Mem_Address, 
                                 unsigned char * pBuffer, 
                                 unsigned char pBufferSize)
{
  I2C_Result Result = I2C_ADD_NOT_EXIST;
  if (I2C__Start() != I2C_SUCCESS) return I2C_BUS_BUSY;
  if (I2C__Write(I2c_Address & 0xFE) == ACK)
    if (I2C__Write(Mem_Address) == ACK)
    {
      while (pBufferSize--) 
        I2C__Write(* pBuffer++);
      Result = I2C_SUCCESS;
    }  
  I2C__Stop(); 
  return Result;   
}

I2C_Result I2C__Write_Register  (unsigned char I2c_Address, 
                                 unsigned char Reg, 
                                 unsigned char data)
{
  I2C_Result Result = I2C_ADD_NOT_EXIST; 
  if (I2C__Start() != I2C_SUCCESS) return I2C_BUS_BUSY;
  if (I2C__Write(I2c_Address & 0xFE) == ACK)
  {
    I2C__Write(Reg);
    I2C__Write(data);
    Result = I2C_SUCCESS;
  }
  I2C__Stop();  
  return Result;
}

void I2C_Delay (void)
{
  unsigned int ucnt = I2C_DelayValue;
  while (ucnt--) asm("nop\n");
}

/* Functions without using enum type */

unsigned char I2C_Write_Page    (unsigned char I2c_Address, 
                                 unsigned char Mem_Address, 
                                 unsigned char * pBuffer, 
                                 unsigned char pBufferSize)
{
  return (unsigned char)I2C__Write_Page(I2c_Address, Mem_Address, 
                                        pBuffer, pBufferSize);
}

unsigned char I2C_Read_Page    (unsigned char I2c_Address, 
                                unsigned char Mem_Address, 
                                unsigned char * pBuffer, 
                                unsigned char pBufferSize)
{
  return (unsigned char)I2C__Read_Page(I2c_Address, Mem_Address, 
                                        pBuffer, pBufferSize);
}